//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Basic4GLMainForm.h"

// Libraries
#include "TomWindowsBasicLib.h"
#include "TomStdBasicLib.h"
#include "TomOpenGLBasicLib.h"
#include "GLBasicLib_gl.h"
#include "GLBasicLib_glu.h"
#include "TomTextBasicLib.h"
#include "TomJoystickBasicLib.h"
#include "TomTrigBasicLib.h"
#include "TomFileIOBasicLib.h"
#include "TomNetBasicLib.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner), m_runMode(RM_STOPPED), m_comp(m_vm), m_glWin(NULL),
    m_glText(NULL)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
    InitLibraries();
    ResetProject();
}
//---------------------------------------------------------------------------
void TMainForm::ResetProject() {

    // Clear out the current project and setup a new basic one with a single
    // source-file.

    // Close existing editors
    for (unsigned int i = 0; i < m_fileEditors.size(); i++)
        delete m_fileEditors[i];
    m_fileEditors.clear();

    // Create new one
    TFileEditor *editor = new TFileEditor(this);
    m_fileEditors.push_back(editor);
    editor->Parent = Tabs;
    editor->Align = alClient;

    // Setup tabs
    SynchroniseTabs();

    // Display the editor
    ShowEditor(0);
}
//---------------------------------------------------------------------------
void TMainForm::ShowEditor(unsigned int index) {

    // Set the correct tab
    Tabs->TabIndex = index;

    // Display the correct editor
    for (unsigned int i = 0; i < m_fileEditors.size(); i++) {
        if (i == index) {
            m_fileEditors[i]->Show();
            m_fileEditors[i]->ActivateEditor();
        }
        else
            m_fileEditors[i]->Hide();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GoStopActionExecute(TObject *Sender)
{
    switch(m_runMode) {
        case RM_STOPPED:
            if (Compile())
                Run();
            break;
        case RM_RUNNING:
        case RM_PAUSED:
            Stop();
            break;
    }
}
//---------------------------------------------------------------------------
bool TMainForm::Compile() {

    // Clear source code from parser
    m_comp.Parser ().SourceCode ().clear ();

    // Reload in from editors
    // Load included files first (if any)
    unsigned int i;
    for (i = 1; i < m_fileEditors.size(); i++)
        LoadParser(m_fileEditors[i]);

    // Load main file last.
    LoadParser(m_fileEditors[0]);

    // Compile
    m_comp.ClearError();
    m_comp.Compile();

    // Return result
    if (m_comp.Error()) {

        // Show error
        PutCursor(m_comp.Line(), m_comp.Col());
        Status->SimpleText = m_comp.GetError().c_str();
        return false;
    }
    return true;
}
//---------------------------------------------------------------------------
void TMainForm::Run() {

    // Run from start

    // Reset virtual machine
    m_vm.Reset();

    // Setup to start program from start
    SetupForRun();

    // Start running
    Continue();
}
//---------------------------------------------------------------------------
void TMainForm::Stop() {

    // Stop program
    m_runMode = RM_STOPPED;
    DeactivateForStop();
    Status->SimpleText = "Program stopped";

    // Update UI
    GoStopAction->ImageIndex = 0;
    GoStopAction->Caption = "Go";
    GoStopAction->Enabled = true;
    GoStopAction->Hint = "Run the program!";

    NewAction->Enabled      = true;
    OpenAction->Enabled     = true;
    SaveAction->Enabled     = CheckModified();
    SaveAsAction->Enabled   = true;
}
//---------------------------------------------------------------------------
void TMainForm::Continue() {
    m_runMode = RM_RUNNING;
    ActivateForContinue();
    Status->SimpleText = "Running...";

    // Update UI
    GoStopAction->ImageIndex = 1;
    GoStopAction->Caption = "Stop";
    GoStopAction->Enabled = true;
    GoStopAction->Hint = "Stop the program!";

    NewAction->Enabled      = false;
    OpenAction->Enabled     = false;
    SaveAction->Enabled     = false;
    SaveAsAction->Enabled   = false;
}
//---------------------------------------------------------------------------
void TMainForm::Pause() {
    m_runMode = RM_PAUSED;
    DeactivateForStop();
    Status->SimpleText = "Program paused";

    // Update UI
    GoStopAction->ImageIndex = 1;
    GoStopAction->Caption = "Stop";
    GoStopAction->Enabled = true;
    GoStopAction->Hint = "Stop the program!";

    NewAction->Enabled      = false;
    OpenAction->Enabled     = false;
    SaveAction->Enabled     = false;
    SaveAsAction->Enabled   = false;
}
//---------------------------------------------------------------------------
void TMainForm::LoadParser(TFileEditor *editor) {

    // Load editor text into parser (appended to bottom)
    for(int i = 0; i < editor->Editor->Lines->Count; i++)
        m_comp.Parser().SourceCode().push_back(editor->Editor->Lines->Strings[i].c_str());
}
//---------------------------------------------------------------------------
void TMainForm::PutCursor(int line, int col) {

    // CODE HERE!!!

}
//---------------------------------------------------------------------------
void TMainForm::SetupForRun() {

    // Setup to run the program from the start.

    // Reset the OpenGL state
    m_glWin->ResetGL();
    m_glWin->Activate();
    m_glWin->OpenGLDefaults();
    m_glWin->SetClosing(false);
}
//---------------------------------------------------------------------------
void TMainForm::ActivateForContinue() {

    // Re-activate to continue the program
    m_glWin->Show();
    m_glWin->Activate();
}
//---------------------------------------------------------------------------
void TMainForm::DeactivateForStop() {

    // Deactivate to stop the program
    if (m_glWin->FullScreen())
        m_glWin->Hide();
}
//---------------------------------------------------------------------------
void TMainForm::PutCursorAtIP() {

    // Find IP
    int line, col;
    m_vm.GetIPInSourceCode (line, col);

    // Put cursor there
    PutCursor(line, col);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
    ShutDownLibraries();
}
//---------------------------------------------------------------------------
void TMainForm::InitLibraries() {

    // Create OpenGL window
    m_glWin     = NULL;
    m_glText    = NULL;

    // Default settings
    bool fullScreen = false, border = true;
    int width = 640, height = 480, bpp = 0;
    ResetGLModeType resetGLMode = RGM_RESETSTATE;

    // Create window
    m_glWin = new glTextGridWindow (
        fullScreen,
        border,
        width,
        height,
        bpp,
        "Basic4GL",
        resetGLMode);

    // Check for errors
    if (m_glWin->Error ()) {
        MessageDlg (
            (AnsiString) m_glWin->GetError().c_str(),
            mtError,
            TMsgDlgButtons() << mbOK,
            0);
        Application->Terminate ();
        return;
    }
    m_glWin->Hide ();

    // Create OpenGL text grid
    m_glText = new glSpriteEngine (
        (ExtractFilePath (Application->ExeName) + "charset.png").c_str (),
        &m_files,
        25,
        40,
        16,
        16);

    // Check for errors
    if (m_glText->Error ()) {
        MessageDlg (
            (AnsiString) + m_glText->GetError ().c_str (),
            mtError,
            TMsgDlgButtons() << mbOK,
            0);
        Application->Terminate ();
        return;
    }
    m_glWin->SetTextGrid (m_glText);

    // Plug in constant and function libraries
    InitTomStdBasicLib      (m_comp);                       // Standard library
    InitTomWindowsBasicLib  (m_comp, &m_files);             // Windows specific library
    InitTomOpenGLBasicLib   (m_comp, m_glWin, &m_files);    // OpenGL
    InitTomTextBasicLib     (m_comp, m_glWin, m_glText);    // Basic text/sprites
    InitGLBasicLib_gl       (m_comp);
    InitGLBasicLib_glu      (m_comp);
    InitTomJoystickBasicLib (m_comp, m_glWin);              // Joystick support
    InitTomTrigBasicLib     (m_comp);                       // Trigonometry library
    InitTomFileIOBasicLib   (m_comp, &m_files);             // File I/O library
    InitTomNetBasicLib      (m_comp);                       // Networking
}
//---------------------------------------------------------------------------
void TMainForm::ShutDownLibraries() {

    // Clear virtual machine state
    m_vm.Clr ();

    // Free text display
    if (m_glText != NULL)
        delete m_glText;

    // Free the openGL window
    if (m_glWin != NULL)
        delete m_glWin;
}
//---------------------------------------------------------------------------
void TMainForm::RunApplication() {
    Show();
    while (!Application->Terminated) {
        DriveVM();
        if (m_runMode == RM_RUNNING)
            Application->ProcessMessages();
        else
            Application->HandleMessage();
    }
}
//---------------------------------------------------------------------------
void TMainForm::DriveVM() {

    // Drive the virtual machine
    if (m_runMode == RM_RUNNING) {

        // Execute a number of VM steps
        try {
            m_vm.Continue(VMSTEPS);
        } catch(...) {
            m_vm.MiscError("An exception occured!");
        }

        // Check for error
        if (m_vm.Error()) {
            PutCursorAtIP();
            Stop();
            Status->SimpleText = m_vm.GetError().c_str();
        }
        else if (m_vm.Done() || m_glWin->Closing()) {
            Stop();
            Status->SimpleText = "Program completed";
            if (m_glWin->Closing() || m_glWin->FullScreen())
                m_glWin->Hide();
        }
    }
}
//---------------------------------------------------------------------------
void TMainForm::SynchroniseTabs() {

    // We always keep at least one editor active
    assert(m_fileEditors.size() > 0);

    // Synchronise tabs with currently open files
    if (m_fileEditors.size() == 1)

        // If just one editor, don't show tabs
        Tabs->Tabs->Clear();

    else {

        // Remove unused tabs
        while (Tabs->Tabs->Count > (int) m_fileEditors.size())
            Tabs->Tabs->Delete(Tabs->Tabs->Count - 1);

        // Add new tabs
        while (Tabs->Tabs->Count < (int) m_fileEditors.size())
            Tabs->Tabs->Add("");

        // Set tab text
        for (unsigned int i = 0; i < m_fileEditors.size(); i++)
            Tabs->Tabs->operator [](i) = m_fileEditors[i]->TabCaption();
    }

    // Set the application caption
    if (m_fileEditors[0]->Filename() == "")
        Application->Title = "Basic4GL, Copyright 2005 Tom Mulgrew";
    else
        Application->Title = m_fileEditors[0]->TabCaption() + " - Basic4GL";
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExitActionExecute(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    Stop();

    // CODE HERE!!!
    // Check whether we need to save changes etc.

    Application->Terminate();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::NewActionExecute(TObject *Sender)
{
    ;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OpenActionExecute(TObject *Sender)
{
    ;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SaveActionExecute(TObject *Sender)
{
    ;
}
//---------------------------------------------------------------------------
bool TMainForm::CheckModified() {

    // CODE HERE!!!
    return true;
}
//---------------------------------------------------------------------------


