// kernel32.lib user32.lib gdi32.lib winspool.lib comdlg32.lib advapi32.lib shell32.lib ole32.lib oleaut32.lib uuid.lib odbc32.lib 

/*	Basic4GL standalone stub.
	Copyright (C) Tom Mulgrew, 2003-2005 (tmulgrew@slingshot.co.nz)

	Contains:
		* Virtual machine
		* Compiled Basic4GL program as op-codes (or will do once it is 
		  externally patched in by Basic4GL.)

	The Basic4GL data is placed in a special segment named "Basic4GL" at the end of the
	executable.
	This simplifies the exe creation code, as it only needs to update the one section
	(and adjust some file size variables).
*/

#include "TomVM.h"
#include "glTextGrid.h"
#include "TomOpenGLBasicLib.h"
#include "glBasicLib_gl.h"
#include "glbasiclib_glu.h" 
#include "TomStdBasicLib.h"
#include "TomWindowsBasicLib.h"
#include "TomTextBasicLib.h"
#include "TomJoystickBasicLib.h"
#include "TomTrigBasicLib.h"
#include "TomFileIOBasicLib.h"
#include "TomNetBasicLib.h"
#include "glSpriteEngine.h"
#include "EmbeddedFiles.h"
#include "Standalone.h"
#include "ResourceIDs.h"

/*
// Allocate space for Basic4GL program
// We only allocate 1 byte, as patcher will resize the program anyway.
#pragma data_seg("Basic4GL")
	char __declspec(allocate("Basic4GL")) basic4GLData[sizeof (StandaloneHeader)];
#pragma data_seg()
*/

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{

	// Allocate objects
    TomVM               vm;											// Virtual machine		
	FileOpener			files;

	// Unpack standalone data
	StandaloneHeader header;

	// Extract the Basic4GL code
	if (vm.Error ()) {
		MessageBox (NULL, "DEBUG1", "DEBUG", MB_OK);
		return 0;
	}

	// Find resource
	HRSRC resInfo = FindResource (NULL, MAKEINTRESOURCE (IDR_BASIC4GLDATA), "BIN");
	if (resInfo == NULL) {
		MessageBox (NULL, "Missing program data", "Program integrity error", MB_OK);
		return 0;
	}

	// Load it
	HGLOBAL resHandle = LoadResource (NULL, resInfo);
	if (resHandle == NULL) {
		MessageBox (NULL, "Missing program data", "Program integrity error", MB_OK);
		return 0;
	}

	// Lock it
	void *basic4GLData = LockResource (resHandle);
	if (basic4GLData == NULL) {
		MessageBox (NULL, "Missing program data", "Program integrity error", MB_OK);
		return 0;
	}

	// Unpack it
	if (!UnpackStandaloneData ((char *) basic4GLData, header, vm, files)) {
		if (vm.Error ())
			MessageBox (NULL, vm.GetError ().c_str (), "Virtual machine stream error", MB_OK);
		else
			MessageBox (NULL, "Program integrity check failed", "Program integrity error", MB_OK);
		return 0;
	}
	if (vm.Error ()) {
		MessageBox (NULL, vm.GetError ().c_str (), "Virtual machine stream error", MB_OK);
		return 0;
	}

	// Create OpenGL window
	int xSize, ySize, bpp;
	switch (header.screenMode) {
	case 1:		xSize = 640;	ySize = 480;	break;
	case 2:		xSize = 800;	ySize = 600;	break;
	case 3:		xSize = 1024;	ySize = 768;	break;
	case 4:		xSize = 1280;	ySize = 1024;	break;
	default:	
		xSize = GetSystemMetrics (SM_CXSCREEN);
		ySize = GetSystemMetrics (SM_CYSCREEN);
	}
	switch (header.colourDepth) {
	case 1:		bpp	= 16; break;
	case 2:		bpp = 32; break;
	default:	bpp = 0;
	}
    glTextGridWindow	glWin (header.fullScreen, header.border, xSize, ySize, bpp, header.title, RGM_RECREATEWINDOW);		// Create window (with text support)

	// Initialise them
    glSpriteEngine		glText("charset.png", &files, 25, 40, 16, 16);	// Setup a text grid for the window
    glWin.SetTextGrid (&glText);									
	if (glText.Error ()) {
		MessageBox (glWin.GetHWND (), glText.GetError ().c_str (), "ERROR!", MB_OK);
		return 0;
	}

	// Initialise function and constant libraries

	// Create dummy compiler object.
	// This is an empty object with no code. It simply acts as a placeholder for the
	// compiler object, to which the function libraries expect to register their functions
	// and constants.
	// The object simply passes the relevant data through to the VM and discards any data
	// that isn't used.
	TomBasicCompiler comp (vm);

    InitTomStdBasicLib (comp);							// Standard library
    InitTomWindowsBasicLib (comp, &files);				// Windows specific library
    InitTomOpenGLBasicLib (comp, &glWin, &files);		// OpenGL
    InitTomTextBasicLib (comp, &glWin, &glText);
    InitGLBasicLib_gl (comp);
    InitGLBasicLib_glu (comp);
	InitTomJoystickBasicLib (comp, &glWin);
    InitTomTrigBasicLib (comp);							// Trigonometry library
    InitTomFileIOBasicLib (comp, &files);				// File I/O library
	InitTomNetBasicLib (comp);

	// Display and initialise OpenGL window
	glWin.Activate ();
	glWin.OpenGLDefaults ();

	// Reset virtual machine
    vm.Reset ();

	try {
		do {
	
			// Execute maximum of 1000 virtual machine Op-codes
			vm.Continue (1000);
	
			// Process windows messages (to keep application responsive)
			glWin.ProcessWindowsMessages();
	
	    } while (!vm.Error () && !vm.Done () && (!(glWin.Closing () && header.escKeyQuits)));
	}
	catch (...) {
		vm.MiscError ("An exception occured");
	}
	
	if (vm.Error ()) {

		// Runtime error encountered.
		// Perform whatever error logic was requested in header file.
		if (header.fullScreen) 
			glWin.Hide ();

		if (header.errorOption == 1)
	
			// Show generic error message
			MessageBox (glWin.GetHWND (), "An error has occured.", "ERROR!", MB_OK);
		else if (header.errorOption == 2) {

			// Show specific error message
			int line, col;
			vm.GetIPInSourceCode (line, col);
			MessageBox (glWin.GetHWND (), ((std::string) "Runtime error: " + vm.GetError () + ", line " + IntToString (line + 1) + ", col " + IntToString (col + 1)).c_str (), "ERROR!", MB_OK);
		}
		// Otherwise just exit
	}
	else if (vm.Done ()) {
		
		// Program completed normally
		// Perform whatever close down logic was requested in header file.
		// (Wait for key, just close e.t.c)

		// Invalid case, fullscreen + wait for window to be closed.		
		if (header.fullScreen && header.finishedOption == 2)
			header.finishedOption = 1;			// Replace with wait for keypress

		if (header.finishedOption == 1) {
			// Wait for keypress or close window
			glWin.ProcessWindowsMessages();
			while (glWin.GetKey () != 0 && !glWin.Closing ()) glWin.ProcessWindowsMessages();
			while (glWin.GetKey () == 0 && !glWin.Closing ()) glWin.ProcessWindowsMessages();
		}
		else if (header.finishedOption == 2) {
			// Wait for window to be closed
			while (!glWin.Closing ())
				glWin.ProcessWindowsMessages ();
		}
		// Otherwise just exit
	}	

//	CleanupTomNetBasicLib ();
	return 0;
}

