/*	Created 29-Feb-2004: Thomas Mulgrew (tmulgrew@slingshot.co.nz)
	Copyright (C) Tom Mulgrew, 2004.

	Structures and routines for standalone Basic4GL programs.

	Standalone programs have all their necessary data patched into the end of the program
	by the standalone .exe generator.
	This module defines the structure of this data, along with the necessary routines to
	unpack it into a useable format.

*/

#include "standalone.h"
#include <windows.h>
#ifndef _MSC_VER
#include <sstream.h>
#endif

bool UnpackStandaloneData (char *rawData, StandaloneHeader& header, TomVM& vm, FileOpener& files) {
	int offset = 0;

// DEBUGGING
//	std::ofstream file ("dump.bin", std::ios::binary | std::ios::out);		// TESTING!!! Dump to stream
//	file.write (rawData, 1000);
//	file.close ();

//MessageBox (NULL, "Read Header", "Milestone", MB_OK);

	// Read header
	memcpy (&header, rawData, sizeof (header));
	offset += sizeof (header);

//MessageBox (NULL, ((std::string) "Offset1 = " + IntToString (offset)).c_str (), "Milestone", MB_OK);

	// Check header validity
	if (header.version != STANDALONE_VERSION)
		return false;

	// Read virtual machine stream
//MessageBox (NULL, "Read Stream data", "Milestone", MB_OK);

	// Read into stream
	int streamSize = *((int *) (rawData + offset));
	offset += sizeof (streamSize);	
//MessageBox (NULL, ((std::string) "Offset2 = " + IntToString (offset)).c_str (), "Milestone", MB_OK);
//MessageBox (NULL, ((std::string) "Stream size = " + IntToString (streamSize)).c_str (), "Milestone", MB_OK);
#ifdef _MSC_VER
	std::strstream stream (rawData + offset, streamSize);
#else
	std::stringstream stream;
	stream.write (rawData + offset, streamSize);
#endif	
	offset += streamSize;

	// Stream into vm
	vm.StreamIn (stream);

//MessageBox (NULL, "Read embedded files", "Milestone", MB_OK);

	// Remaining data is embedded files
	files.AddFiles (rawData, offset);
	return true;
}
